@@ -614,7 +614,7 @@ draw_window (GdkDrawable * drawable, GtkWidget * widget, NetkWindow * win,
 
     if ((winrect->width > 2) && (winrect->height > 2))
     {
-	gdk_draw_rectangle (drawable,
+        gdk_draw_rectangle (drawable,
                             is_active ?
                             widget->style->bg_gc[GTK_STATE_SELECTED] :
                             widget->style->bg_gc[GTK_STATE_NORMAL],
@@ -698,7 +698,7 @@ draw_window (GdkDrawable * drawable, GtkWidget * widget, NetkWindow * win,
 
     if ((winrect->width > 0) && (winrect->height > 0))
     {
-	gdk_draw_rectangle (drawable,
+        gdk_draw_rectangle (drawable,
                             is_active ?
                             widget->style->fg_gc[GTK_STATE_SELECTED] :
                             widget->style->fg_gc[GTK_STATE_NORMAL],
@@ -1177,25 +1177,36 @@ netk_pager_button_release (GtkWidget * widget, GdkEventButton * event)
         i = workspace_at_point (pager, event->x, event->y, &viewport_x,
                                 &viewport_y);
 
-        if (i >= 0)
+        if (i >= 0 && (space = netk_screen_get_workspace (pager->priv->screen, i)))
         {
-            space = netk_screen_get_workspace (pager->priv->screen, i);
-
-            if (space)
-            {
+	    int screen_width, screen_height;
+	    
+	    /* Don't switch the desktop if we're already there */		
+            if (space != netk_screen_get_active_workspace (pager->priv->screen))
                 netk_workspace_activate (space);
 
                 /* EWMH only lets us move the viewport for the active workspace,
                  * but we just go ahead and hackily assume that the activate
                  * just above takes effect prior to moving the viewport
                  */
-                netk_screen_move_viewport (pager->priv->screen,
-                                           viewport_x, viewport_y);
 
-                if (pager->priv->drag_window)
-                    netk_window_activate (pager->priv->drag_window);
-            }
-        }
+            	/* Transform the pointer location to viewport origin, assuming
+	         * that we want the nearest "regular" viewport containing the
+		 * pointer.
+		 */				   
+		screen_width  = netk_screen_get_width  (pager->priv->screen);
+	        screen_height = netk_screen_get_height (pager->priv->screen);
+		viewport_x = (viewport_x / screen_width)  * screen_width;
+		viewport_y = (viewport_y / screen_height) * screen_height;
+				
+		if (netk_workspace_get_viewport_x (space) != viewport_x ||
+	            netk_workspace_get_viewport_y (space) != viewport_y)
+		  netk_screen_move_viewport (pager->priv->screen, viewport_x, viewport_y);
+
+		if ((space == netk_screen_get_active_workspace (pager->priv->screen))
+	             && (pager->priv->drag_window))
+		  netk_window_activate (pager->priv->drag_window);
+	}
 
         if (pager->priv->drag_window)
             netk_pager_clear_drag (pager);
