@@ -56,7 +56,7 @@ typedef struct _NetkTaskClass NetkTaskClass;
 
 #define DEFAULT_WIDTH 1
 #define DEFAULT_HEIGHT 48
-#define MAX_BUTTON_WIDTH 200
+#define DEFAULT_BUTTON_WIDTH 200
 
 #define TIMEOUT_ACTIVATE 1000
 
@@ -791,12 +791,14 @@ netk_tasklist_size_request (GtkWidget * widget, GtkRequisition * requisition)
         gtk_widget_size_request (task->button, &child_req);
 
         max_height = MAX (child_req.height, max_height);
-        max_width = MAX (child_req.width, max_width);
 
         l = l->next;
     }
 
-    max_width = MIN (max_width, MAX_BUTTON_WIDTH);
+    if (tasklist->priv->show_label)
+        max_width = DEFAULT_BUTTON_WIDTH;
+    else
+        max_width = max_height;
     
     tasklist->priv->max_button_width = max_width;
     tasklist->priv->max_button_height = max_height;
@@ -1296,40 +1298,41 @@ static gboolean
 netk_tasklist_include_window (NetkTasklist *tasklist, NetkWindow *win)          
 {                                                                               
     NetkWorkspace *active_workspace;                                              
+
     if (netk_window_get_state (win) & NETK_WINDOW_STATE_SKIP_TASKLIST)            
     {
         return FALSE;                                                               
     }
 
-    if (tasklist->priv->include_all_workspaces)                                   
+    if (tasklist->priv->include_all_workspaces)
     {
-        return TRUE;                                                                
+        return TRUE;
     }
 
-    if (netk_window_is_pinned (win))                                              
+    if (netk_window_is_pinned (win))
     {
-        return TRUE;                                                                
+        return TRUE;
     }
 
     active_workspace = netk_screen_get_active_workspace (tasklist->priv->screen); 
 
-    if (active_workspace == NULL)                                                 
+    if (active_workspace == NULL)
     {
-        return TRUE;                                                                
+        return TRUE;
     }
 
-    if (active_workspace != netk_window_get_workspace (win))                      
+    if (active_workspace != netk_window_get_workspace (win))
     {
-        return FALSE;                                                               
+        return FALSE;
     }
 
-    if (!netk_workspace_is_virtual (active_workspace))                            
+    if (!netk_workspace_is_virtual (active_workspace))
     {
-        return TRUE;                                                                
+        return TRUE;
     }
 
-    return netk_window_is_in_viewport (win, active_workspace);                    
-}                                                                               
+    return netk_window_is_in_viewport (win, active_workspace);
+}
 
 static void
 netk_tasklist_update_lists (NetkTasklist * tasklist)
@@ -1352,8 +1355,7 @@ netk_tasklist_update_lists (NetkTasklist * tasklist)
         if (netk_tasklist_include_window (tasklist, win))
         {
             win_task = netk_task_new_from_window (tasklist, win);
-            tasklist->priv->windows =
-                g_list_prepend (tasklist->priv->windows, win_task);
+            tasklist->priv->windows = g_list_prepend (tasklist->priv->windows, win_task);
             g_hash_table_insert (tasklist->priv->win_hash, win, win_task);
 
             gtk_widget_set_parent (win_task->button, GTK_WIDGET (tasklist));
@@ -1476,8 +1478,7 @@ netk_tasklist_update_icon_geometries (NetkTasklist * tasklist)
         if (!GTK_WIDGET_REALIZED (task->button))
             continue;
 
-        gdk_window_get_origin (GTK_BUTTON (task->button)->event_window, &x,
-                               &y);
+        gdk_window_get_origin (GTK_BUTTON (task->button)->event_window, &x, &y);
         width = task->button->allocation.width;
         height = task->button->allocation.height;
 
@@ -1546,44 +1547,44 @@ netk_tasklist_window_changed_workspace (NetkWindow * window,
     }
 }
 
-static gboolean                                                                 
-do_netk_tasklist_update_lists (gpointer data)                                   
-{                                                                               
-    NetkTasklist *tasklist = NETK_TASKLIST (data);                                
-    tasklist->priv->idle_callback_tag = 0;                                        
-    netk_tasklist_update_lists (tasklist);                                        
-    return FALSE;                                                                 
-}                                                                               
+static gboolean
+do_netk_tasklist_update_lists (gpointer data)
+{
+    NetkTasklist *tasklist = NETK_TASKLIST (data);
+    tasklist->priv->idle_callback_tag = 0;
+    netk_tasklist_update_lists (tasklist);
+    return FALSE;
+}
 
-static void                                                                     
-netk_tasklist_window_changed_geometry (NetkWindow   *window,                    
-                                       NetkTasklist *tasklist)                  
-{                                                                               
-    NetkTask *win_task;                                                           
-    gboolean show;                                                                
-    if (tasklist->priv->idle_callback_tag != 0)                                   
+static void
+netk_tasklist_window_changed_geometry (NetkWindow   *window,
+                                       NetkTasklist *tasklist)
+{
+    NetkTask *win_task;
+    gboolean show;
+    if (tasklist->priv->idle_callback_tag != 0)
     {
-        return;                                                                     
+        return;
     }
     
-    /*                                                                            
-     * We want to re-generate the task list if                                    
-     * the window is shown but shouldn't be or                                    
-     * the window isn't shown but should be.                                      
-     */                                                                           
+    /*
+     * We want to re-generate the task list if
+     * the window is shown but shouldn't be or
+     * the window isn't shown but should be.
+     */
 
-    win_task = g_hash_table_lookup (tasklist->priv->win_hash, window);            
-    show = netk_tasklist_include_window(tasklist, window);                        
-    if ((win_task == NULL && !show) || (win_task != NULL && show))                
+    win_task = g_hash_table_lookup (tasklist->priv->win_hash, window);
+    show = netk_tasklist_include_window(tasklist, window);
+    if ((win_task == NULL && !show) || (win_task != NULL && show))
     {
-        return;                                                                     
+        return;
     }
     
-    /* Don't keep any stale references */                                         
+    /* Don't keep any stale references */
     gtk_widget_queue_draw (GTK_WIDGET (tasklist));
     tasklist->priv->idle_callback_tag = 
                 g_idle_add (do_netk_tasklist_update_lists, tasklist);
-}                                                                               
+}
 
 static void
 netk_tasklist_connect_window (NetkTasklist * tasklist, NetkWindow * window)
