@@ -30,6 +30,7 @@
 #include "netk-xutils.h"
 #include "netk-private.h"
 
+#if 0  /* unused? */
 static void
 set_type_realize_handler (GtkWidget * widget, gpointer data)
 {
@@ -39,6 +40,7 @@ set_type_realize_handler (GtkWidget * widget, gpointer data)
 
     p_netk_set_type_hint (GDK_WINDOW_XWINDOW (widget->window), (NetkWindowType) GPOINTER_TO_INT(data));
 }
+#endif
 
 void
 netk_gtk_window_set_type (GtkWindow * window, NetkWindowType type)
@@ -46,56 +48,53 @@ netk_gtk_window_set_type (GtkWindow * window, NetkWindowType type)
     g_return_if_fail (window != NULL);
     
     /* GTK 2.4.x has properties for that */
-    if ((gtk_major_version >=2) && (gtk_minor_version >= 3))
+#if GTK_CHECK_VERSION(2, 2, 0)
+    switch (type)
     {
-        switch (type)
-        {
-            case NETK_WINDOW_DESKTOP:
-                DBG ("type NETK_WINDOW_DESKTOP");
-                g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_DESKTOP, NULL);
-                break;
-            case NETK_WINDOW_DOCK:
-                DBG ("type NETK_WINDOW_DOCK");
-                g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_DOCK, NULL);
-                break;
-            case NETK_WINDOW_MODAL_DIALOG:
-                DBG ("type NETK_WINDOW_MODAL_DIALOG");
-                g_object_set (G_OBJECT(window), "modal", TRUE, NULL);
-            case NETK_WINDOW_DIALOG:
-                DBG ("type NETK_WINDOW_DIALOG");
-                g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_DIALOG, NULL);
-                break;
-            case NETK_WINDOW_TOOLBAR:
-                DBG ("type NETK_WINDOW_TOOLBAR");
-                g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_TOOLBAR, NULL);
-                break;
-            case NETK_WINDOW_MENU:
-                DBG ("type NETK_WINDOW_MENU");
-                g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_MENU, NULL);
-                break;
-            case NETK_WINDOW_UTILITY:
-                DBG ("type NETK_WINDOW_UTILITY");
-                g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_UTILITY, NULL);
-                break;
-            case NETK_WINDOW_SPLASHSCREEN:
-                DBG ("type NETK_WINDOW_SPLASHSCREEN");
-                g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_SPLASHSCREEN, NULL);
-                break;
-            default:
-                DBG ("default, using NETK_WINDOW_NORMAL");
-                g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_NORMAL, NULL);
-                break;
-        }
+        case NETK_WINDOW_DESKTOP:
+            DBG ("type NETK_WINDOW_DESKTOP");
+            g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_DESKTOP, NULL);
+            break;
+        case NETK_WINDOW_DOCK:
+            DBG ("type NETK_WINDOW_DOCK");
+            g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_DOCK, NULL);
+            break;
+        case NETK_WINDOW_MODAL_DIALOG:
+            DBG ("type NETK_WINDOW_MODAL_DIALOG");
+            g_object_set (G_OBJECT(window), "modal", TRUE, NULL);
+        case NETK_WINDOW_DIALOG:
+            DBG ("type NETK_WINDOW_DIALOG");
+            g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_DIALOG, NULL);
+            break;
+        case NETK_WINDOW_TOOLBAR:
+            DBG ("type NETK_WINDOW_TOOLBAR");
+            g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_TOOLBAR, NULL);
+            break;
+        case NETK_WINDOW_MENU:
+            DBG ("type NETK_WINDOW_MENU");
+            g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_MENU, NULL);
+            break;
+        case NETK_WINDOW_UTILITY:
+            DBG ("type NETK_WINDOW_UTILITY");
+            g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_UTILITY, NULL);
+            break;
+        case NETK_WINDOW_SPLASHSCREEN:
+            DBG ("type NETK_WINDOW_SPLASHSCREEN");
+            g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_SPLASHSCREEN, NULL);
+            break;
+        default:
+            DBG ("default, using NETK_WINDOW_NORMAL");
+            g_object_set (G_OBJECT(window), "type-hint", GDK_WINDOW_TYPE_HINT_NORMAL, NULL);
+            break;
     }
-    else
+#else    
+    if (GTK_WIDGET_REALIZED (window))
     {
-        if (GTK_WIDGET_REALIZED (window))
-        {
-            set_type_realize_handler (GTK_WIDGET (window), GINT_TO_POINTER(type));
-        }
-        g_signal_connect (GTK_WINDOW (window), "realize",
-                          G_CALLBACK (set_type_realize_handler), GINT_TO_POINTER(type));
+        set_type_realize_handler (GTK_WIDGET (window), GINT_TO_POINTER(type));
     }
+    g_signal_connect (GTK_WINDOW (window), "realize",
+                      G_CALLBACK (set_type_realize_handler), GINT_TO_POINTER(type));
+#endif
 }
 
 static void
@@ -105,7 +104,7 @@ set_no_input_realize_handler (GtkWidget * widget, gpointer data)
     Atom *protocols = None;
     int n;
 
-	g_return_if_fail (widget != NULL);
+        g_return_if_fail (widget != NULL);
     g_return_if_fail (GDK_IS_WINDOW(widget->window));
 
     wmhints =
@@ -113,7 +112,7 @@ set_no_input_realize_handler (GtkWidget * widget, gpointer data)
                      GDK_WINDOW_XWINDOW (widget->window));
     
     if (!wmhints && !(wmhints = XAllocWMHints()))
-	return;
+        return;
     
     wmhints->flags |= InputHint;
     wmhints->input = False;
