@@ -45,7 +45,7 @@
 
 static GHashTable *window_hash = NULL;
 
-/* Keep 0-7 in sync with the numbers in the WindowState enum. Yeah I'm
+/* Keep 0-9 in sync with the numbers in the WindowState enum. Yeah I'm
  * a loser.
  */
 #define COMPRESS_STATE(window)                          \
@@ -1472,7 +1472,6 @@ update_state (NetkWindow * window)
         case NETK_WINDOW_SPLASHSCREEN:
             window->priv->skip_taskbar = TRUE;
             break;
-
         case NETK_WINDOW_TOOLBAR:
         case NETK_WINDOW_MENU:
         case NETK_WINDOW_UTILITY:
@@ -1481,10 +1480,11 @@ update_state (NetkWindow * window)
             /* Skip taskbar if the window is transient
              * for some main application window
              */
-            if (window->priv->transient_for)
+            if (window->priv->transient_for != None)
+            {
                 window->priv->skip_taskbar = TRUE;
+            }
             break;
-
         case NETK_WINDOW_NORMAL:
             break;
     }
@@ -1865,10 +1865,18 @@ force_update_now (NetkWindow * window)
     get_icons (window);
 
     new_state = COMPRESS_STATE (window);
-
     if (old_state != new_state)
-        emit_state_changed (window, old_state ^ new_state, new_state);
+    {
+        NetkWindowState changed_state = old_state ^ new_state;
 
+        emit_state_changed (window, changed_state, new_state);
+        /* Hack, pwack, arck, crack...!!! */
+        if ((changed_state & NETK_WINDOW_STATE_SKIP_TASKLIST) &&
+            (old_state & NETK_WINDOW_STATE_SKIP_TASKLIST))
+        {
+            emit_geometry_changed (window);
+        }
+    }
     if (old_actions != window->priv->actions)
         emit_actions_changed (window, old_actions ^ window->priv->actions,
                               window->priv->actions);
