@@ -59,6 +59,7 @@ typedef struct
 	
 	GdkScreen *gscreen;
 	
+	gboolean use_svg;
 	GHashTable *icons;
 	
 	GList *search_paths;
@@ -103,6 +104,7 @@ static void xfce_icon_theme_finalize(GObject *object);
 
 static guint icon_theme_signals[XFCEIC_N_SIGNALS] = { 0 };
 static GObjectClass *parent_class = NULL;
+static gboolean _gdk_pixbuf_has_svg = FALSE;
 
 static const gchar *builtin_icon_categories[][XFCE_N_BUILTIN_ICON_CATEGORIES] =
 {
@@ -132,7 +134,7 @@ static gchar *fallback_themes[] =
 };
 
 /* list in order of preference */
-static const gchar *icon_ext[] =
+static const gchar *icon_ext_without_svg[] =
 {
 	".png", ".xpm", NULL
 };
@@ -140,7 +142,6 @@ static const gchar *icon_ext_with_svg[] =
 {
 	".svg", ".svgz", ".png", ".xpm", NULL
 };
-static gchar **supported_icon_ext = NULL;
 
 G_INLINE_FUNC void
 xfce_icon_theme_category_list_free(XfceIconThemeCategoryList *catlist)
@@ -177,7 +178,7 @@ theme_add_all(XfceIconThemeSingleton *singleton, const gchar *base_path,
 	gchar buf[PATH_MAX], **theme_dirs, *filepath, *p, *icon_name, **inherits;
 	XfceIcon *xicon;
 	GDir *dir;
-	const gchar *file, *type;
+	const gchar *file, *type, **icon_ext;
 	gint icon_size, j, k;
 	GList *l;
 	struct stat st;
@@ -190,6 +191,11 @@ theme_add_all(XfceIconThemeSingleton *singleton, const gchar *base_path,
 	g_hash_table_insert(singleton->hash_for_stupid_themes, g_strdup(buf),
 			(gpointer)0xdeadbeef);
 	
+	if(singleton->use_svg)
+		icon_ext = icon_ext_with_svg;
+	else
+		icon_ext = icon_ext_without_svg;
+	
 	themerc = xfce_rc_simple_open(buf, TRUE);
 	if(themerc) {
 		if(xfce_rc_has_group(themerc, "Icon Theme")) {
@@ -238,8 +244,8 @@ theme_add_all(XfceIconThemeSingleton *singleton, const gchar *base_path,
 					while((file = g_dir_read_name(dir))) {
 						gboolean is_icon_file = FALSE;
 						
-						for(k = 0; supported_icon_ext[k]; k++) {
-							if(g_str_has_suffix(file, supported_icon_ext[k])) {
+						for(k = 0; icon_ext[k]; k++) {
+							if(g_str_has_suffix(file, icon_ext[k])) {
 								is_icon_file = TRUE;
 								break;
 							}
@@ -373,10 +379,8 @@ static void
 xfce_icon_theme_class_init(XfceIconThemeClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS(klass);
-	GSList *pix_formats, *l;
-	gchar **pix_mimetypes;
-	gint i;
-	
+	GdkPixbuf *svg_test;
+		
 	parent_class = g_type_class_peek_parent(klass);
 	
 	gobject_class->finalize = xfce_icon_theme_finalize;
@@ -406,23 +410,16 @@ xfce_icon_theme_class_init(XfceIconThemeClass *klass)
 	}
 	
 	/* check SVG format support */
-	pix_formats = gdk_pixbuf_get_formats();
-	for(l = pix_formats; l; l = l->next) {
-		GdkPixbufFormat *fmt = l->data;
-		pix_mimetypes = gdk_pixbuf_format_get_mime_types(fmt);
-		for(i = 0; pix_mimetypes[i]; i++) {
-			if(!g_ascii_strcasecmp(pix_mimetypes[i], "image/svg")) {
-				supported_icon_ext = (gchar **)icon_ext_with_svg;
-				break;
-			}
-		}
-		g_strfreev(pix_mimetypes);
+	svg_test = gdk_pixbuf_new_from_file(DATADIR "/xfce4/xfce-svg-test.svg",
+			NULL);
+	if(svg_test) {
+		_gdk_pixbuf_has_svg = TRUE;
+		g_object_unref(G_OBJECT(svg_test));
+		DBG("SVG support enabled!");
+	} else {
+		_gdk_pixbuf_has_svg = FALSE;
+		DBG("SVG support disabled!");
 	}
-	if(pix_formats)
-		g_slist_free(pix_formats);
-	
-	if(!supported_icon_ext)
-		supported_icon_ext = (gchar **)icon_ext;
 }
 
 static void
@@ -494,6 +491,9 @@ xfce_icon_theme_get_for_screen(GdkScreen *screen)
 	if(!screen)
 		screen = gdk_display_get_default_screen(gdk_display_get_default());
 	
+	/* create this first so we've definitely inited _gdk_pixbuf_has_svg */
+	icon_theme = g_object_new(XFCE_TYPE_ICON_THEME, NULL);
+	
 	singleton = g_object_get_data(G_OBJECT(screen), "xfce-icon-theme");
 	if(!singleton) {
 		gchar **paths, *kde_icon_dir = NULL, *homeicons;
@@ -505,9 +505,7 @@ xfce_icon_theme_get_for_screen(GdkScreen *screen)
 		singleton->refcnt = 1;
 		singleton->gscreen = screen;
 		singleton->next_custom_id = XFCE_N_BUILTIN_ICON_CATEGORIES;
-		
-		homeicons = g_strconcat(xfce_get_homedir(), "/.icons/", NULL);
-		singleton->search_paths = g_list_append(singleton->search_paths, homeicons);
+		singleton->use_svg = _gdk_pixbuf_has_svg;
 		
 		paths = xfce_resource_lookup_all(XFCE_RESOURCE_DATA, "icons/");
 		for(i = 0; paths[i]; i++) {
@@ -524,18 +522,18 @@ xfce_icon_theme_get_for_screen(GdkScreen *screen)
 					g_build_path(G_DIR_SEPARATOR_S, DATADIR, "icons/", NULL));
 		}
 		
-		/* FIXME: not sure if we need this next line or not */
-		/*gtk_icon_theme_prepend_search_path(icon_theme, DATADIR "/xfce4/icons");*/
 		if(kdedir && *kdedir && strcmp(kdedir, "/usr") && strcmp(kdedir, PREFIX)) {
 			kde_icon_dir = g_strconcat(kdedir, "/share/icons/", NULL);
 			singleton->search_paths = g_list_append(singleton->search_paths, kde_icon_dir);
 		}
 		
+		homeicons = g_strconcat(xfce_get_homedir(), "/.icons/", NULL);
+		singleton->search_paths = g_list_prepend(singleton->search_paths, homeicons);
+		
 		g_object_set_data(G_OBJECT(screen), "xfce-icon-theme", singleton);
 	} else
 		singleton->refcnt++;
 	
-	icon_theme = g_object_new(XFCE_TYPE_ICON_THEME, NULL);
 	icon_theme->priv->gscreen = screen;
 	icon_theme->priv->singleton = singleton;
 	
@@ -574,19 +572,26 @@ check_icon_theme(XfceIconTheme *icon_theme)
 }
 
 static gchar *
-search_fallback_paths(const gchar *icon_name, gint icon_size)
+search_fallback_paths(XfceIconThemeSingleton *singleton, const gchar *icon_name,
+		gint icon_size)
 {
 	gchar *filename = NULL, buf[PATH_MAX];
 	const gchar *default_fallback_path = "/usr/share/pixmaps/";
 	const gchar *xfce_prefixed_fallback_path = DATADIR "/pixmaps/";
 	gint i, path_len;
+	const gchar **icon_ext;
 	
 	g_strlcpy(buf, xfce_prefixed_fallback_path, PATH_MAX);
 	g_strlcat(buf, icon_name, PATH_MAX);
 	path_len = strlen(buf);
 	
-	for(i = 0; supported_icon_ext[i]; i++) {
-		g_strlcat(buf, supported_icon_ext[i], PATH_MAX);
+	if(singleton->use_svg)
+		icon_ext = icon_ext_with_svg;
+	else
+		icon_ext = icon_ext_without_svg;
+	
+	for(i = 0; icon_ext[i]; i++) {
+		g_strlcat(buf, icon_ext[i], PATH_MAX);
 		if(g_file_test(buf, G_FILE_TEST_IS_REGULAR|G_FILE_TEST_IS_SYMLINK)) {
 			filename = g_strdup(buf);
 			break;
@@ -599,8 +604,8 @@ search_fallback_paths(const gchar *icon_name, gint icon_size)
 		g_strlcat(buf, icon_name, PATH_MAX);
 		path_len = strlen(buf);
 		
-		for(i = 0; supported_icon_ext[i]; i++) {
-			g_strlcat(buf, supported_icon_ext[i], PATH_MAX);
+		for(i = 0; icon_ext[i]; i++) {
+			g_strlcat(buf, icon_ext[i], PATH_MAX);
 			if(g_file_test(buf, G_FILE_TEST_IS_REGULAR|G_FILE_TEST_IS_SYMLINK)) {
 				filename = g_strdup(buf);
 				break;
@@ -622,12 +627,12 @@ enum {
 };
 
 static gchar *
-lookup_icon (XfceIconTheme *icon_theme, const gchar *icon_name, 
-		gint icon_size, int *quality)
+lookup_icon(XfceIconTheme *icon_theme, const gchar *icon_name, 
+		gint icon_size, gint *quality)
 {
 	XfceIconThemeSingleton *singleton;
 	GList *icons, *l;
-	XfceIcon *xicon, *fuzzy_xicon = NULL;
+	XfceIcon *xicon, *closest_xicon = NULL;
 	gchar *filename = NULL, *p, *new_icon_name = NULL;
 	
 	g_return_val_if_fail(XFCE_IS_ICON_THEME(icon_theme) && icon_name, NULL);
@@ -639,7 +644,7 @@ lookup_icon (XfceIconTheme *icon_theme, const gchar *icon_name,
 	
 	/* chop off file extension */
 	p = g_strrstr(icon_name, ".");
-	if(p && strlen(p) < 5)
+	if(p && strlen(p) < 6)
 		new_icon_name = g_strndup(icon_name, strlen(icon_name)-strlen(p));
 	if(new_icon_name)
 		icon_name = (const gchar *)new_icon_name;
@@ -647,60 +652,42 @@ lookup_icon (XfceIconTheme *icon_theme, const gchar *icon_name,
 	check_icon_theme(icon_theme);
 		
 	icons = g_hash_table_lookup(singleton->icons, icon_name);
-	if(!icons) {
-		filename = search_fallback_paths(icon_name, icon_size);
-		if(new_icon_name)
-			g_free(new_icon_name);
-		if (quality != NULL)
-			*quality = XFCE_ICON_FALLBACK;
-		return filename;
-	}
-	
 	for(l = icons; l; l = l->next) {
 		xicon = l->data;
-		if(xicon->size == -1) {
-			/* if we have a scalable (svg) image, take it unconditionally */
-			filename = xicon->path;
-			break;
+		
+		if(xicon->size == icon_size && xicon->is_in_theme) {
+			/* non-scalable icon of the right size, prefer in-theme */
+			closest_xicon = xicon;
+		} else if(xicon->size == -1 && (!closest_xicon 
+				|| (xicon->is_in_theme && !closest_xicon->is_in_theme)
+				|| (closest_xicon->size != icon_size
+					&& closest_xicon->size != -1)))
+		{
+			/* svg icon in the theme, and we don't already have a non-scalable
+			 * non-svg in the proper size */
+			closest_xicon = xicon;
 		} else {
-			/* if not, go through and attempt to find the closest match */
-			gint this_diff, cur_closest_diff = 1000;
-			
-			this_diff = icon_size - xicon->size;
-			if(fuzzy_xicon)
-				cur_closest_diff = icon_size - fuzzy_xicon->size;
-			
-			/* if:  this is a closer size match AND we're not replacing an
-			 * in-theme icon with an out-of-theme icon AND we're not using a
-			 * too-small icon below the threshold, pick it as a candidate */
-			if(this_diff < cur_closest_diff
-					&& (!fuzzy_xicon || !fuzzy_xicon->is_in_theme)
-					&& this_diff <= SIZE_THRESHOLD_BELOW)
+			/* try to find a "close enough" match */
+			gint cur_diff = closest_xicon ? icon_size - closest_xicon->size : 1000;
+			gint new_diff = icon_size - xicon->size;
+			if((xicon->is_in_theme || !closest_xicon || !closest_xicon->is_in_theme)
+					&& new_diff <= SIZE_THRESHOLD_BELOW
+					&& new_diff < cur_diff)
 			{
-				fuzzy_xicon = xicon;
+				closest_xicon = xicon;
 			}
 		}
 	}
 	
-	if(filename) {
-		if (quality != NULL)
-			*quality = xicon->is_in_theme ? XFCE_ICON_IN_THEME :
-				XFCE_ICON_INHERITED;
-		
-		filename = g_strdup(filename);
-	}
-	else if(fuzzy_xicon) {
-		if (quality != NULL)
-			*quality = fuzzy_xicon->is_in_theme ? XFCE_ICON_IN_THEME :
-				XFCE_ICON_INHERITED;
-		
-		filename = g_strdup(fuzzy_xicon->path);
-	}
-	else {
-		if (quality != NULL)
+	if(closest_xicon) {
+		if(quality)
+			*quality = closest_xicon->is_in_theme ? XFCE_ICON_IN_THEME :
+					XFCE_ICON_INHERITED;
+		filename = g_strdup(closest_xicon->path);
+	} else {
+		if(quality)
 			*quality = XFCE_ICON_FALLBACK;
-		
-		filename = search_fallback_paths(icon_name, icon_size);
+		filename = search_fallback_paths(singleton, icon_name, icon_size);
 	}
 	
 	if(new_icon_name)
@@ -728,7 +715,7 @@ gchar *
 xfce_icon_theme_lookup(XfceIconTheme *icon_theme, const gchar *icon_name,
 		gint icon_size)
 {
-	return lookup_icon (icon_theme, icon_name, icon_size, NULL);
+	return lookup_icon(icon_theme, icon_name, icon_size, NULL);
 }
 
 /**
@@ -753,7 +740,7 @@ xfce_icon_theme_lookup_list(XfceIconTheme *icon_theme, GList *icon_names,
 	XfceIconThemePriv *priv;
 	GList *l;
 	gchar *filename = NULL, *lookup = NULL;
-	int quality = XFCE_ICON_FALLBACK - 1, lookup_quality;
+	gint quality = XFCE_ICON_FALLBACK - 1, lookup_quality;
 	
 	g_return_val_if_fail(XFCE_IS_ICON_THEME(icon_theme) && icon_names, NULL);
 	priv = icon_theme->priv;
@@ -762,19 +749,17 @@ xfce_icon_theme_lookup_list(XfceIconTheme *icon_theme, GList *icon_names,
 		lookup = lookup_icon(icon_theme, (gchar *)l->data, icon_size, 
 				&lookup_quality);
 
-		if (lookup) {
-			if (lookup_quality == XFCE_ICON_IN_THEME) {
-				g_free (filename);
+		if(lookup) {
+			if(lookup_quality == XFCE_ICON_IN_THEME) {
+				g_free(filename);
 				filename = lookup;
 				break;
-			}
-			else if (lookup_quality > quality) {
-				g_free (filename);
+			} else if(lookup_quality > quality) {
+				g_free(filename);
 				filename = lookup;
 				quality = lookup_quality;
-			}
-			else
-				g_free (lookup);
+			} else
+				g_free(lookup);
 		}
 	}
 	
@@ -863,9 +848,11 @@ xfce_icon_theme_load(XfceIconTheme *icon_theme, const gchar *icon_name,
 	g_return_val_if_fail(XFCE_IS_ICON_THEME(icon_theme) && icon_name, NULL);
 	
 	filename = xfce_icon_theme_lookup(icon_theme, icon_name, icon_size);
-	if(filename)
+	if(filename) {
 		pix = xfce_pixbuf_new_from_file_at_size(filename, icon_size,
 				icon_size, NULL);
+		g_free(filename);
+	}
 	
 	return pix;
 }
@@ -895,9 +882,11 @@ xfce_icon_theme_load_list(XfceIconTheme *icon_theme, GList *icon_names,
 	g_return_val_if_fail(XFCE_IS_ICON_THEME(icon_theme) && icon_names, NULL);
 	
 	filename = xfce_icon_theme_lookup_list(icon_theme, icon_names, icon_size);
-	if(filename)
+	if(filename) {
 		pix = xfce_pixbuf_new_from_file_at_size(filename, icon_size,
 				icon_size, NULL);
+		g_free(filename);
+	}
 	
 	return pix;
 }
@@ -929,9 +918,11 @@ xfce_icon_theme_load_category(XfceIconTheme *icon_theme,
 	g_return_val_if_fail(XFCE_IS_ICON_THEME(icon_theme), NULL);
 	
 	filename = xfce_icon_theme_lookup_category(icon_theme, category, icon_size);
-	if(filename)
+	if(filename) {
 		pix = xfce_pixbuf_new_from_file_at_size(filename, icon_size,
 				icon_size, NULL);
+		g_free(filename);
+	}
 	
 	return pix;
 }
@@ -1089,6 +1080,8 @@ xfce_icon_theme_register_category(XfceIconTheme *icon_theme, GList *icon_names)
  * Unregisters @category, which must have been registered previously with
  * xfce_icon_theme_register_category().  After this call, the category may no
  * longer be used for icon lookups.
+ *
+ * Since 4.1
  **/
 void
 xfce_icon_theme_unregister_category(XfceIconTheme *icon_theme,
@@ -1108,3 +1101,45 @@ xfce_icon_theme_unregister_category(XfceIconTheme *icon_theme,
 		old_list->data = NULL;
 	}
 }
+
+/**
+ * xfce_icon_theme_set_use_svg:
+ * @icon_theme: An #XfceIconTheme.
+ * @use_svg: Whether or not to use SVG icons.
+ *
+ * Sets whether or not to include SVG icons in icon lookups and loads.  This
+ * option defaults to %TRUE.
+ *
+ * Since 4.1.91
+ **/
+void
+xfce_icon_theme_set_use_svg(XfceIconTheme *icon_theme, gboolean use_svg)
+{
+	g_return_if_fail(XFCE_IS_ICON_THEME(icon_theme)
+			&& icon_theme->priv->singleton);
+	
+	if(use_svg && !_gdk_pixbuf_has_svg)
+		return;
+	
+	if(use_svg != icon_theme->priv->singleton->use_svg) {
+		icon_theme->priv->singleton->use_svg = use_svg;
+		invalidate_theme(icon_theme, icon_theme->priv->singleton);
+	}
+}
+
+/**
+ * xfce_icon_theme_get_use_svg:
+ * @icon_theme: An #XfceIconTheme.
+ *
+ * Reports whether or not the icon theme is set to return SVG icons.
+ *
+ * Since 4.1.91
+ **/
+gboolean
+xfce_icon_theme_get_use_svg(XfceIconTheme *icon_theme)
+{
+	g_return_val_if_fail(XFCE_IS_ICON_THEME(icon_theme)
+			&& icon_theme->priv->singleton, TRUE);
+	
+	return icon_theme->priv->singleton->use_svg;
+}
